# `MapView` Component API

## Props

| Prop                          | Type                                                                                                                                                                               | Default         | Note                                                                                                                                                                                                                                                                                                                                                                                                           |
| ----------------------------- | ---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------- | --------------- | -------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------- |
| `provider`                    | \[ &#124; `` `google `` \]                                                                                                                                                         |                 | To specify the map framework to use. <br/><br/>Pass `` `google `` for GoogleMaps, otherwise omit the prop to use the native map framework (`MapKit` in iOS and `GoogleMaps` in android).                                                                                                                                                                                                                       |
| `region`                      | `Region.t`                                                                                                                                                                         |                 | The region to be displayed by the map. <br/><br/>The region is defined by the center coordinates and the span of coordinates to display.                                                                                                                                                                                                                                                                       |
| `initialRegion`               | `Region.t`                                                                                                                                                                         |                 | The initial region to be displayed by the map. Use this prop instead of `region` only if you don't want to control the viewport of the map besides the initial region.<br/><br/> Changing this prop after the component has mounted will not result in a region change.<br/><br/> This is similar to the `initialValue` prop of a text input.                                                                  |
| `camera`                      | `camera`                                                                                                                                                                           |                 | The camera view the map should display. If you use this, the `region` property is ignored.                                                                                                                                                                                                                                                                                                                     |
| `initialCamera`               | `camera`                                                                                                                                                                           |                 | Like `initialRegion`, use this prop instead of `camera` only if you don't want to control the viewport of the map besides the initial camera setting.<br/><br/> Changing this prop after the component has mounted will not result in a region change.<br/><br/> This is similar to the `initialValue` prop of a text input.                                                                                   |
| `mapPadding`                  | `edgePadding`                                                                                                                                                                      |                 | Adds custom padding to each side of the map. Useful when map elements/markers are obscured. **Note** Google Maps only.                                                                                                                                                                                                                                                                                         |
| `paddingAdjustmentBehavior`   | \[ <br/> &#124; `` `always `` <br/> &#124; `` `automatic `` <br/> &#124; `` `never `` \]                                                                                           | `` `never ``    | Indicates how/when to affect padding with safe area insets (`GoogleMaps` in iOS only)                                                                                                                                                                                                                                                                                                                          |
| `liteMode`                    | `bool`                                                                                                                                                                             | `false`         | Enable lite mode. **Note**: Android only.                                                                                                                                                                                                                                                                                                                                                                      |
| `mapType`                     | \[ <br/> &#124; `` `standard `` <br/> &#124; `` `satellite `` <br/> &#124; `` `hybrid `` <br/> &#124; `` `none `` <br/> &#124; `` `terrain `` <br/> &#124; `` `mutedStandard `` \] | `` `standard `` | The map type to be displayed. <br/><br/> - `` `standard ``: standard road map <br/> - `` `none ``: no map (Not available on MapKit) <br/> - `` `satellite ``: satellite view <br/> - `` `hybrid ``: satellite view with roads and points of interest overlayed <br/> - `` `terrain ``: topographic view (Android only)<br/> - `` `mutedStandard ``: more subtle, makes markers/lines pop more (iOS 11.0+ only) |
| `customMapStyle`              | `array(Js.Json.t)`                                                                                                                                                                 |                 | Adds custom styling to the map component. See [README](https://github.com/react-native-community/react-native-maps#customizing-the-map-style) for more information.                                                                                                                                                                                                                                            |
| `showsUserLocation`           | `bool`                                                                                                                                                                             | `false`         | If `true` the app will ask for the user's location. **NOTE**: You need to add `NSLocationWhenInUseUsageDescription` key in Info.plist to enable geolocation, otherwise it is going to _fail silently_! You will also need to add an explanation for why you need the users location against `NSLocationWhenInUseUsageDescription` in Info.plist. Otherwise Apple may reject your app submission.               |
| `userLocationAnnotationTitle` | `string`                                                                                                                                                                           |                 | The title of the annotation for current user location. This only works if `showsUserLocation` is true. There is a default value `My Location` set by MapView. **Note**: iOS only.                                                                                                                                                                                                                              |
| `followsUserLocation`         | `bool`                                                                                                                                                                             | `false`         | If `true` the map will focus on the user's location. This only works if `showsUserLocation` is true and the user has shared their location. **Note**: iOS only.                                                                                                                                                                                                                                                |
| `showsMyLocationButton`       | `bool`                                                                                                                                                                             | `true`          | If `false` hide the button to move map to the current user's location.                                                                                                                                                                                                                                                                                                                                         |
| `showsPointsOfInterest`       | `bool`                                                                                                                                                                             | `true`          | If `false` points of interest won't be displayed on the map.                                                                                                                                                                                                                                                                                                                                                   |
| `showsCompass`                | `bool`                                                                                                                                                                             | `true`          | If `false` compass won't be displayed on the map.                                                                                                                                                                                                                                                                                                                                                              |
| `showsScale`                  | `bool`                                                                                                                                                                             | `true`          | A Boolean indicating whether the map shows scale information. **Note**: Apple Maps only.                                                                                                                                                                                                                                                                                                                       |
| `showsBuildings`              | `bool`                                                                                                                                                                             | `true`          | A Boolean indicating whether the map displays extruded building information.                                                                                                                                                                                                                                                                                                                                   |
| `showsTraffic`                | `bool`                                                                                                                                                                             | `true`          | A Boolean value indicating whether the map displays traffic information.                                                                                                                                                                                                                                                                                                                                       |
| `showsIndoors`                | `bool`                                                                                                                                                                             | `true`          | A Boolean indicating whether indoor maps should be enabled.                                                                                                                                                                                                                                                                                                                                                    |
| `showsIndoorLevelPicker`      | `bool`                                                                                                                                                                             | `false`         | A Boolean indicating whether indoor level picker should be enabled. **Note:** Google Maps only (either Android or iOS with `PROVIDER_GOOGLE`).                                                                                                                                                                                                                                                                 |
| `zoomEnabled`                 | `bool`                                                                                                                                                                             | `true`          | If `false` the user won't be able to pinch/zoom the map.                                                                                                                                                                                                                                                                                                                                                       |
| `zoomTapEnabled`              | `bool`                                                                                                                                                                             | `true`          | If `false` the user won't be able to double tap to zoom the map. **Note:** But it will greatly decrease delay of tap gesture recognition. **Note:** Google Maps on iOS only                                                                                                                                                                                                                                    |
| `zoomControlEnabled`          | `bool`                                                                                                                                                                             | `true`          | If `false` the zoom control at the bottom right of the map won't be visible **Note:** Android only.                                                                                                                                                                                                                                                                                                            |
| `minZoomLevel`                | `int`                                                                                                                                                                              | `0`             | Minimum zoom value for the map, must be between 0 and 20                                                                                                                                                                                                                                                                                                                                                       |
| `maxZoomLevel`                | `int`                                                                                                                                                                              | `20`            | Maximum zoom value for the map, must be between 0 and 20                                                                                                                                                                                                                                                                                                                                                       |
| `rotateEnabled`               | `bool`                                                                                                                                                                             | `true`          | If `false` the user won't be able to pinch/rotate the map.                                                                                                                                                                                                                                                                                                                                                     |
| `scrollEnabled`               | `bool`                                                                                                                                                                             | `true`          | If `false` the user won't be able to change the map region being displayed.                                                                                                                                                                                                                                                                                                                                    |
| `pitchEnabled`                | `bool`                                                                                                                                                                             | `true`          | If `false` the user won't be able to adjust the camera’s pitch angle.                                                                                                                                                                                                                                                                                                                                          |
| `toolbarEnabled`              | `bool`                                                                                                                                                                             | `true`          | `Android only` If `false` will hide 'Navigate' and 'Open in Maps' buttons on marker press                                                                                                                                                                                                                                                                                                                      |
| `cacheEnabled`                | `bool`                                                                                                                                                                             | `false`         | If `true` map will be cached and displayed as an image instead of being interactable, for performance usage. **Note:** Apple Maps only                                                                                                                                                                                                                                                                         |
| `loadingEnabled`              | `bool`                                                                                                                                                                             | `false`         | If `true` a loading indicator will show while the map is loading.                                                                                                                                                                                                                                                                                                                                              |
| `loadingIndicatorColor`       | `ReactNative.Color.t`                                                                                                                                                              | `"#606060"`     | Sets loading indicator color, default to `#606060`.                                                                                                                                                                                                                                                                                                                                                            |
| `loadingBackgroundColor`      | `ReactNative.Color.t`                                                                                                                                                              | `"#FFFFFF"`     | Sets loading background color, default to `#FFFFFF`.                                                                                                                                                                                                                                                                                                                                                           |
| `moveOnMarkerPress`           | `bool`                                                                                                                                                                             | `true`          | `Android only` If `false` the map won't move when a marker is pressed.                                                                                                                                                                                                                                                                                                                                         |
| `legalLabelInsets`            | `ReactNative.View.edgeInsets`                                                                                                                                                      |                 | If set, changes the position of the "Legal" label link from the OS default. **Note:** iOS only.                                                                                                                                                                                                                                                                                                                |
| `kmlSrc`                      | `string`                                                                                                                                                                           |                 | The URL from KML file. **Note:** Google Maps and Markers only (either Android or iOS with `PROVIDER_GOOGLE`).                                                                                                                                                                                                                                                                                                  |
| `compassOffset`               | `point`                                                                                                                                                                            |                 | If set, changes the position of the compass. **Note:** iOS Maps only.                                                                                                                                                                                                                                                                                                                                          |

Please also see
[Reason React Native documentation of `View` props](https://reasonml-community.github.io/reason-react-native/en/docs/components/View/)
for additional supported props.

## Events

Events return either no data or an appropriate instance of the parametrised type
`ReactNative.Event.syntheticEvent('a)`. Accordingly, to handle events you need
to pass functions of type `unit => unit` or
`ReactNative.Event.syntheticEvent('a) => unit` where `'a` should be of the type
specified in the table below. In the latter case, you will need to use the `##`
accessor for each individual key within the function specified. For example:
`onPress={e => Console.log(e##nativeEvent)}`.

| Event Name                | `'a` (if applicable)          | Notes                                                                                                                                                                                                                                                         |
| ------------------------- | ----------------------------- | ------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------- |
| `onCalloutPress`          |                               | Callback that is called when a callout is tapped by the user.                                                                                                                                                                                                 |
| `onMapReady`              |                               | Callback that is called once the map is fully loaded.                                                                                                                                                                                                         |
| `onKmlReady`              | `kmlContainer`                | Callback that is called once the kml is fully loaded.                                                                                                                                                                                                         |
| `onRegionChange`          | `Region.t`                    | Callback that is called continuously when the region changes, such as when a user is dragging the map.                                                                                                                                                        |
| `onRegionChangeComplete`  | `Region.t`                    | Callback that is called once when the region changes, such as when the user is done moving the map.                                                                                                                                                           |
| `onUserLocationChange`    | `{. "coordinate": location }` | Callback that is called when the underlying map figures our users current location (coordinate also includes isFromMockProvider value for Android API 18 and above). Make sure **showsUserLocation** is set to _true_ and that the provider is `` `google ``. |
| `onPress`                 | `copos`                       | Callback that is called when user taps on the map.                                                                                                                                                                                                            |
| `onDoublePress`           | `copos`                       | Callback that is called when user double taps on the map.                                                                                                                                                                                                     |
| `onPanDrag`               | `copos`                       | Callback that is called when user presses and drags the map. **NOTE**: for iOS `scrollEnabled` should be set to `false` to trigger the event                                                                                                                  |
| `onPoiClick`              | `poi`                         | Callback that is called when user click on a POI.                                                                                                                                                                                                             |
| `onLongPress`             | `copos`                       | Callback that is called when user makes a "long press" somewhere on the map.                                                                                                                                                                                  |
| `onMarkerPress`           |                               | Callback that is called when a marker on the map is tapped by the user.                                                                                                                                                                                       |
| `onMarkerSelect`          |                               | Callback that is called when a marker on the map becomes selected. This will be called when the callout for that marker is about to be shown. **Note**: iOS only.                                                                                             |
| `onMarkerDeselect`        |                               | Callback that is called when a marker on the map becomes deselected. This will be called when the callout for that marker is about to be hidden. **Note**: iOS only.                                                                                          |
| `onMarkerDragStart`       | `copos`                       | Callback that is called when the user initiates a drag on a marker (if it is draggable)                                                                                                                                                                       |
| `onMarkerDrag`            | `copos`                       | Callback called continuously as a marker is dragged                                                                                                                                                                                                           |
| `onMarkerDragEnd`         | `copos`                       | Callback that is called when a drag on a marker finishes. This is usually the point you will want to setState on the marker's coordinate again                                                                                                                |
| `onIndoorLevelActivated`  | `indoorLevel`                 | Callback that is called when a level on indoor building is activated                                                                                                                                                                                          |
| `onIndoorBuildingFocused` | `indoorBuilding`              | Callback that is called when a indoor building is focused/unfocused                                                                                                                                                                                           |

## Methods

Deprecated methods are not supported. All methods require the `MapView` element
as the first argument. For additional arguments please refer to the table below.

| Method Name                 | Types of Additional Arguments        | Returns                                                          | Notes                                                                                                                                                                                                                                                                                                          |
| --------------------------- | ------------------------------------ | ---------------------------------------------------------------- | -------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------- |
| `getCamera`                 | unit                                 | `camera`                                                         | Returns the current camera configuration.                                                                                                                                                                                                                                                                      |
| `animateCamera`             | `camera`,<br/> `duration`            | unit                                                             | Animate the camera to a new view. In JS, you can pass a partial camera object here; any property not given will remain unmodified, however, that is not supported in these bindings and a camera object is required. Do note that any optional arguments not specified in the new object will not be modified. |
| `setCamera`                 | `camera`, <br/>`duration`            | unit                                                             | Like `animateCamera`, but sets the new view instantly, without an animation.                                                                                                                                                                                                                                   |
| `animateToRegion`           | `Region.t`, `float`                  | unit                                                             | duration of the animation should be specified as a `float`                                                                                                                                                                                                                                                     |
| `getMapBoundaries`          | unit                                 | `Js.Promise.t(mapBoundaries)`                                    | Returns current boundaries of the map                                                                                                                                                                                                                                                                          |
| `setMapBoundaries`          | `mapBoundaries`                      | unit                                                             | The boundary is defined by the map's center coordinates, not the device's viewport itself. **Note:** Google Maps only.                                                                                                                                                                                         |
| `setIndoorActiveLevelIndex` | `int`                                | unit                                                             | `levelIndex` should be specified as an `int`                                                                                                                                                                                                                                                                   |
| `fitToElements`             | `bool`                               | unit                                                             | Can specify whether the transition should be animated by the additonal argument                                                                                                                                                                                                                                |
| `fitToSuppliedMarkers`      | `array(string)`, <br/> `fitConfig`   | unit                                                             | markerIDs should be specified as `array(string)` If you need to use this in `ComponentDidMount`, make sure you put it in a timeout or it will cause performance problems.                                                                                                                                      |
| `fitToCoordinates`          | `array(LatLng.t)`, <br/> `fitConfig` | unit                                                             | Coordinates should be specified as an array of `LatLng.t`. Options may also be specified. If called in `ComponentDidMount` in android, it will cause an exception. It is recommended to be called from the MapView `onLayout` event.                                                                           |
| `pointForCoordinate`        | `LatLng.t`                           | `Js.Promise.t(point)`                                            | Converts a map coordinate (specified as `LatLng.t`) to a view coordinate (`point`).                                                                                                                                                                                                                            |
| `coordinateForPoint`        | `point`                              | `Js.Promise.t(LatLng.t)`                                         | Converts a view coordinate (`point`) to a map coordinate (specified as `LatLng.t`).                                                                                                                                                                                                                            |
| `getMarkersFrames`          | `bool`                               | `Js.Promise.t({. "markerID": {. point: point, frame: frame } })` | Get markers' centers and frames in view coordinates, can limit to only visible markers by the additional argument. **Note**: iOS only.                                                                                                                                                                         |

## Types

### `camera`

```reason
camera:
  (
    ~center: LatLng.t,
    ~pitch: float,
    ~heading: float,
    ~altitude: float=?,
    ~zoom: int=?,
    unit
  ) =>
  camera
```

Note that `altitude` is only for when `MapKit` is used on iOS and `zoom` is only
for use with Google Maps. Specification of height differs between MapKit on iOS
and Google Maps differ. For a cross-platform app, it is necessary to specify
both the zoom level and the altitude separately.

### `location`

```reason
type location = {
  .
  "latitude": float,
  "longitude": float,
  "altitude": float,
  "timestamp": float,
  "accuracy": float,
  "altitudeAccuracy": float,
  "speed": float,
};
```

`timestamp` is milliseconds elapsed since the epoch

### `duration`

```reason
duration: (~duration: float) => duration
```

### `fitConfig`

```reason
fitConfig: (~edgePadding: edgePadding, ~animated: bool) => fitConfig
```

### `frame`

```reason
type frame = {
  .
  "x": float,
  "y": float,
  "width": float,
  "height": float,
};
```

### `edgePadding`

```reason
edgePadding:
  (~top: float, ~right: float, ~bottom: float, ~left: float) => edgePadding
```

### `mapBoundaries`

```reason
type mapBoundaries = {
  .
  "northEast": LatLng.t,
  "southWest": LatLng.t,
}
```

```reason
mapBoundaries:
  (~northEast: LatLng.t, ~southWest: LatLng.t) =>
  mapBoundaries
```

### `marker`

```reason
marker:
  (~id: string, ~coordinate: LatLng.t, ~title: string, ~description: string) =>
  marker
```

### `kmlContainer`

```reason
type kmlContainer = {. "markers": array(marker)}
```

### `indoorBuilding`

```reason
type indoorBuilding = {
  .
  "underground": bool,
  "activeLevelIndex": int,
  "levels": array(indoorLevel),
}
```

### `indoorLevel`

```reason
type indoorLevel = {
  .
  "activeLevelIndex": int,
  "name": string,
  "shortName": string,
}
```
